@@ -12,6 +12,8 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import os
+
 from cursive import exception as cursive_exception
 import glance_store
 from glance_store import backend
@@ -73,14 +75,23 @@ class ImageDataController(object):
         :param image: The image will be restored
         :param staging_store: The store used for staging
         """
-        loc = glance_store.location.get_location_from_uri(str(
-            CONF.node_staging_uri + '/' + image.image_id))
-        try:
-            staging_store.delete(loc)
-        except glance_store.exceptions.NotFound:
-            pass
-        finally:
-            self._restore(image_repo, image)
+        # NOTE(abhishek): staging_store not being used in this function
+        # because of bug #1803498
+        # TODO(abhishek): refactor to use the staging_store when the
+        # "Rethinking Filesystem Access" spec is implemented in Train
+        file_path = str(CONF.node_staging_uri + '/' + image.image_id)[7:]
+        if os.path.exists(file_path):
+            try:
+                os.unlink(file_path)
+            except OSError as e:
+                LOG.error(_("Cannot delete staged image data %(fn)s "
+                            "[Errno %(en)d]"), {'fn': file_path,
+                                                'en': e.errno})
+        else:
+            LOG.warning(_("Staged image data not found "
+                          "at %(fn)s"), {'fn': file_path})
+
+        self._restore(image_repo, image)
 
     def _delete(self, image_repo, image):
         """Delete the image.
@@ -100,6 +111,18 @@ class ImageDataController(object):
 
     @utils.mutating
     def upload(self, req, image_id, data, size):
+        backend = None
+        if CONF.enabled_backends:
+            backend = req.headers.get('x-image-meta-store',
+                                      CONF.glance_store.default_backend)
+
+            try:
+                glance_store.get_store_from_store_identifier(backend)
+            except glance_store.UnknownScheme as exc:
+                raise webob.exc.HTTPBadRequest(explanation=exc.msg,
+                                               request=req,
+                                               content_type='text/plain')
+
         image_repo = self.gateway.get_repo(req.context)
         image = None
         refresher = None
@@ -129,7 +152,7 @@ class ImageDataController(object):
                                  encodeutils.exception_to_unicode(e))
 
                 image_repo.save(image, from_state='queued')
-                image.set_data(data, size)
+                image.set_data(data, size, backend=backend)
 
                 try:
                     image_repo.save(image, from_state='saving')
@@ -274,9 +297,16 @@ class ImageDataController(object):
         # NOTE(jokke): this is horrible way to do it but as long as
         # glance_store is in a shape it is, the only way. Don't hold me
         # accountable for it.
+        # TODO(abhishekk): After removal of backend module from glance_store
+        # need to change this to use multi_backend module.
         def _build_staging_store():
             conf = cfg.ConfigOpts()
-            backend.register_opts(conf)
+
+            try:
+                backend.register_opts(conf)
+            except cfg.DuplicateOptError:
+                pass
+
             conf.set_override('filesystem_store_datadir',
                               CONF.node_staging_uri[7:],
                               group='glance_store')
@@ -382,8 +412,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         return {'size': image_size, 'data': request.body_file}
 
     def stage(self, request):
-        if not CONF.enable_image_import:
-            msg = _("Image import is not supported at this site.")
+        if "glance-direct" not in CONF.enabled_import_methods:
+            msg = _("'glance-direct' method is not available at this site.")
             raise webob.exc.HTTPNotFound(explanation=msg)
         try:
             request.get_content_type(('application/octet-stream',))
