@@ -53,6 +53,12 @@ from glance import i18n
 from glance.i18n import _, _LE, _LI, _LW
 
 
+try:
+    from webob.acceptparse import AcceptLanguageValidHeader  # noqa
+    USING_WEBOB_1_8 = True
+except ImportError:
+    USING_WEBOB_1_8 = False
+
 bind_opts = [
     cfg.HostAddressOpt('bind_host',
                        default='0.0.0.0',
@@ -311,6 +317,13 @@ wsgi_opts = [
                       '"HTTP_X_FORWARDED_PROTO".')),
 ]
 
+store_opts = [
+    cfg.DictOpt('enabled_backends',
+                help=_('Key:Value pair of store identifier and store type. '
+                       'In case of multiple backends should be separated'
+                       'using comma.')),
+]
+
 
 LOG = logging.getLogger(__name__)
 
@@ -319,6 +332,7 @@ CONF.register_opts(bind_opts)
 CONF.register_opts(socket_opts)
 CONF.register_opts(eventlet_opts)
 CONF.register_opts(wsgi_opts)
+CONF.register_opts(store_opts)
 profiler_opts.set_defaults(CONF)
 
 ASYNC_EVENTLET_THREAD_POOL_LIST = []
@@ -442,6 +456,13 @@ def initialize_glance_store():
     glance_store.verify_default_store()
 
 
+def initialize_multi_store():
+    """Initialize glance multi store backends."""
+    glance_store.register_store_opts(CONF)
+    glance_store.create_multi_stores(CONF)
+    glance_store.verify_store()
+
+
 def get_asynchronous_eventlet_pool(size=1000):
     """Return eventlet pool to caller.
 
@@ -593,7 +614,10 @@ class Server(object):
         self.client_socket_timeout = CONF.client_socket_timeout or None
         self.configure_socket(old_conf, has_changed)
         if self.initialize_glance_store:
-            initialize_glance_store()
+            if CONF.enabled_backends:
+                initialize_multi_store()
+            else:
+                initialize_glance_store()
 
     def reload(self):
         """
@@ -1009,8 +1033,10 @@ class Request(webob.Request):
     def best_match_content_type(self):
         """Determine the requested response content-type."""
         supported = ('application/json',)
-        bm = self.accept.best_match(supported)
-        return bm or 'application/json'
+        best_matches = self.accept.acceptable_offers(supported)
+        if not best_matches:
+            return 'application/json'
+        return best_matches[0][0]
 
     def get_content_type(self, allowed_content_types):
         """Determine content type of the request body."""
@@ -1024,7 +1050,7 @@ class Request(webob.Request):
         else:
             return content_type
 
-    def best_match_language(self):
+    def _best_match_language_1_7(self):
         """Determines best available locale from the Accept-Language header.
 
         :returns: the best language match or None if the 'Accept-Language'
@@ -1035,6 +1061,29 @@ class Request(webob.Request):
         langs = i18n.get_available_languages('glance')
         return self.accept_language.best_match(langs)
 
+    def _best_match_language_1_8(self):
+        """Determines best available locale from the Accept-Language header.
+
+        :returns: the best language match or None if the 'Accept-Language'
+                  header was not available in the request.
+        """
+        if not self.accept_language:
+            return None
+        langs = i18n.get_available_languages('glance')
+        # NOTE(rosmaita): give the webob lookup() function a sentinal value
+        # for default so we can preserve the behavior of this function as
+        # indicated by the current unit tests.  See Launchpad bug #1765748.
+        best_match = self.accept_language.lookup(langs, default='fake_LANG')
+        if best_match == 'fake_LANG':
+            best_match = None
+        return best_match
+
+    def best_match_language(self):
+        if USING_WEBOB_1_8:
+            return self._best_match_language_1_8()
+        else:
+            return self._best_match_language_1_7()
+
     def get_range_from_request(self, image_size):
         """Return the `Range` in a request."""
 
