@@ -48,7 +48,8 @@ def _import_delayed_delete():
 
 class ImageFactory(object):
     _readonly_properties = ['created_at', 'updated_at', 'status', 'checksum',
-                            'size', 'virtual_size']
+                            'os_hash_algo', 'os_hash_value', 'size',
+                            'virtual_size']
     _reserved_properties = ['owner', 'locations', 'deleted', 'deleted_at',
                             'direct_url', 'self', 'file', 'schema']
 
@@ -71,7 +72,8 @@ class ImageFactory(object):
     def new_image(self, image_id=None, name=None, visibility='shared',
                   min_disk=0, min_ram=0, protected=False, owner=None,
                   disk_format=None, container_format=None,
-                  extra_properties=None, tags=None, **other_args):
+                  extra_properties=None, tags=None, os_hidden=False,
+                  **other_args):
         extra_properties = extra_properties or {}
         self._check_readonly(other_args)
         self._check_unexpected(other_args)
@@ -89,6 +91,7 @@ class ImageFactory(object):
                      min_ram=min_ram, protected=protected,
                      owner=owner, disk_format=disk_format,
                      container_format=container_format,
+                     os_hidden=os_hidden,
                      extra_properties=extra_properties, tags=tags or [])
 
 
@@ -107,7 +110,7 @@ class Image(object):
         'importing': ('active', 'deleted', 'queued'),
         'active': ('pending_delete', 'deleted', 'deactivated'),
         'killed': ('deleted',),
-        'pending_delete': ('deleted',),
+        'pending_delete': ('deleted', 'active'),
         'deleted': (),
         'deactivated': ('active', 'deleted'),
     }
@@ -119,11 +122,14 @@ class Image(object):
         self.updated_at = updated_at
         self.name = kwargs.pop('name', None)
         self.visibility = kwargs.pop('visibility', 'shared')
+        self.os_hidden = kwargs.pop('os_hidden', False)
         self.min_disk = kwargs.pop('min_disk', 0)
         self.min_ram = kwargs.pop('min_ram', 0)
         self.protected = kwargs.pop('protected', False)
         self.locations = kwargs.pop('locations', [])
         self.checksum = kwargs.pop('checksum', None)
+        self.os_hash_algo = kwargs.pop('os_hash_algo', None)
+        self.os_hash_value = kwargs.pop('os_hash_value', None)
         self.owner = kwargs.pop('owner', None)
         self._disk_format = kwargs.pop('disk_format', None)
         self._container_format = kwargs.pop('container_format', None)
@@ -194,7 +200,8 @@ class Image(object):
 
     @container_format.setter
     def container_format(self, value):
-        if hasattr(self, '_container_format') and self.status != 'queued':
+        if (hasattr(self, '_container_format') and
+                self.status not in ('queued', 'importing')):
             msg = _("Attribute container_format can be only replaced "
                     "for a queued image.")
             raise exception.Forbidden(message=msg)
@@ -206,7 +213,8 @@ class Image(object):
 
     @disk_format.setter
     def disk_format(self, value):
-        if hasattr(self, '_disk_format') and self.status != 'queued':
+        if (hasattr(self, '_disk_format') and
+                self.status not in ('queued', 'importing')):
             msg = _("Attribute disk_format can be only replaced "
                     "for a queued image.")
             raise exception.Forbidden(message=msg)
@@ -275,7 +283,7 @@ class Image(object):
     def get_data(self, *args, **kwargs):
         raise NotImplementedError()
 
-    def set_data(self, data, size=None):
+    def set_data(self, data, size=None, backend=None):
         raise NotImplementedError()
 
 
@@ -292,9 +300,9 @@ class ExtraProperties(collections.MutableMapping, dict):
 
     def __eq__(self, other):
         if isinstance(other, ExtraProperties):
-            return dict(self).__eq__(dict(other))
+            return dict.__eq__(self, dict(other))
         elif isinstance(other, dict):
-            return dict(self).__eq__(other)
+            return dict.__eq__(self, other)
         else:
             return False
 
@@ -302,10 +310,10 @@ class ExtraProperties(collections.MutableMapping, dict):
         return not self.__eq__(other)
 
     def __len__(self):
-        return dict(self).__len__()
+        return dict.__len__(self)
 
     def keys(self):
-        return dict(self).keys()
+        return dict.keys(self)
 
 
 class ImageMembership(object):
