@@ -13,8 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import webob.exc
-
 import glance.api.v2.discovery
 import glance.tests.unit.utils as unit_test_utils
 import glance.tests.utils as test_utils
@@ -26,20 +24,19 @@ class TestInfoControllers(test_utils.BaseTestCase):
         super(TestInfoControllers, self).setUp()
         self.controller = glance.api.v2.discovery.InfoController()
 
-    def test_get_import_info_when_import_not_enabled(self):
-        """When import not enabled, should return 404 just like v2.5"""
-        self.config(enable_image_import=False)
+    def test_get_import_info_with_empty_method_list(self):
+        """When methods list is empty, should still return import methods"""
+        self.config(enabled_import_methods=[])
         req = unit_test_utils.get_fake_request()
-        self.assertRaises(webob.exc.HTTPNotFound,
-                          self.controller.get_image_import,
-                          req)
+        output = self.controller.get_image_import(req)
+        self.assertIn('import-methods', output)
+        self.assertEqual([], output['import-methods']['value'])
 
     def test_get_import_info(self):
         # TODO(rosmaita): change this when import methods are
         # listed in the config file
         import_methods = ['glance-direct', 'web-download']
 
-        self.config(enable_image_import=True)
         req = unit_test_utils.get_fake_request()
         output = self.controller.get_image_import(req)
         self.assertIn('import-methods', output)
